#define PI 3.14159265

uniform float amount;

varying vec4 color;
varying vec2 imageCoord;

uniform vec2 resolution;
uniform vec2 textureSize;
uniform vec2 imageSize;
uniform sampler2D sampler0;

vec2 img2tex( vec2 v ) { return v / textureSize * imageSize; }

vec3 trim( vec3 c, vec2 p )
{
	if( min(p.x,p.y) < 0.0 || max(p.x,p.y) > 1.0 )
		c = vec3(0.0);
	return c;
}

void main()
{
	float prop = resolution.x / resolution.y;

	vec2 p = vec2( imageCoord.x, imageCoord.y * resolution.y / resolution.x );
	vec2 m = vec2( 0.5, 0.5 / prop );
	vec2 d = p - m;

	float r = sqrt(dot(d,d));
	float power = PI / sqrt(dot(m,m)) * amount;

	float bind;
	if( power > 0.0 )
		bind = sqrt(dot(m,m));
	else
		bind = prop < 1.0 ? m.x : m.y;

	vec2 uv;
	if( power > 0.0 )
		// fisheye
		uv = m + normalize(d) * tan(r * power) * bind / tan(bind * power);
	else if( power < 0.0 )
		// antifisheye
		uv = m + normalize(d) * atan(r * power) * bind / atan(bind * power);
	else
		uv = p;
	uv.y *= prop;

	vec3 col = texture2D( sampler0, img2tex(uv) ).rgb;
	col = trim( col, uv );

	gl_FragColor = vec4( col, 1.0 ) * color;
}