#version 120

// ported from https://www.shadertoy.com/view/XssGz8

uniform float amountBarrel = 0.;
uniform float amountZoom = 0.;

varying vec2 imageCoord;
varying vec2 textureCoord;
uniform vec2 resolution;
uniform vec2 textureSize;
uniform vec2 imageSize;
uniform sampler2D sampler0;
uniform float time;

vec2 remap( vec2 t, vec2 a, vec2 b ) {
	return clamp( (t - a) / (b - a), 0.0, 1.0 );
}

vec3 spectrum_offset_rgb( float t )
{
    float t0 = 3.0 * t - 1.5;
	vec3 ret = clamp( vec3( -t0, 1.0-abs(t0), t0), 0.0, 1.0);
    
    return ret;
}

vec2 brownConradyDistortion(vec2 uv, float dist)
{
    uv.x = uv.x * 2.0 - 1.0;
    uv.y = uv.y * 2.0 - 1.0;
    float barrelDistortion1 = 0.1 * dist;
    float barrelDistortion2 = -0.025 * dist;

    float r2 = dot(uv,uv);
    uv *= 1.0 + barrelDistortion1 * r2 + barrelDistortion2 * r2 * r2;
    
    uv.x = uv.x * 0.5 + 0.5;
    uv.y = uv.y * 0.5 + 0.5;

    return uv;
}

vec2 distort( vec2 uv, float t, vec2 min_distort, vec2 max_distort )
{
    vec2 dist = mix( min_distort, max_distort, t );
    
    return brownConradyDistortion( uv, 75.0 * dist.x );

    // return barrelDistortion( uv,1.75*dist );

}

float nrand( vec2 n )
{
	return fract(sin(dot(n.xy, vec2(12.9898, 78.233)))* 43758.5453);
}

vec3 render( vec2 uv )
{
    return texture2D(sampler0, uv / textureSize.xy * imageSize.xy).rgb;
}

void main() {
	vec2 uv = imageCoord;

  // glsl is hard
  float scale = 0.0017 * resolution.y / 1280.0;
  // float scale = 1.0;
    
  const float MAX_DIST_PX = 50.0;
  float max_distort_px = MAX_DIST_PX * amountBarrel * scale;
	vec2 max_distort = vec2(max_distort_px);
  vec2 min_distort = 0.5 * max_distort;
    
  
  const int num_iter = 7;
  const float stepsiz = 1.0 / (float(num_iter)-1.0);
  float rnd = nrand( uv + fract(time) );
  float t = rnd*stepsiz;
  
  vec3 sumcol = vec3(0.0);
	vec3 sumw = vec3(0.0);
	for ( int i=0; i<num_iter; ++i )
	{
		vec3 w = spectrum_offset_rgb( t );
    sumw += w;
    vec2 uvd = distort(uv, t, min_distort, max_distort);
		sumcol += w * render( uvd );
    t += stepsiz;
	}
  sumcol.rgb /= sumw;
    
  vec3 outcol = sumcol.rgb;
  outcol =  outcol;
  outcol += rnd/255.0;
    
	gl_FragColor = vec4( outcol, 1.0);
}
