#version 120
#define PI 3.14159265

varying vec2 imageCoord;

uniform vec2 textureSize;
uniform vec2 imageSize;

uniform sampler2D sampler0;

uniform float zoom;
uniform vec2 xy;

// https://www.shadertoy.com/view/XdcXzn

uniform float brightness = 0.;
uniform float contrast = 1.;
uniform float saturation = 1.;

mat4 brightnessMatrix( float brightness )
{
    return mat4( 1, 0, 0, 0,
                 0, 1, 0, 0,
                 0, 0, 1, 0,
                 brightness, brightness, brightness, 1 );
}

mat4 contrastMatrix( float contrast )
{
	float t = ( 1.0 - contrast ) / 2.0;
    
    return mat4( contrast, 0, 0, 0,
                 0, contrast, 0, 0,
                 0, 0, contrast, 0,
                 t, t, t, 1 );

}

mat4 saturationMatrix( float saturation )
{
    vec3 luminance = vec3( 0.3086, 0.6094, 0.0820 );
    
    float oneMinusSat = 1.0 - saturation;
    
    vec3 red = vec3( luminance.x * oneMinusSat );
    red+= vec3( saturation, 0, 0 );
    
    vec3 green = vec3( luminance.y * oneMinusSat );
    green += vec3( 0, saturation, 0 );
    
    vec3 blue = vec3( luminance.z * oneMinusSat );
    blue += vec3( 0, 0, saturation );
    
    return mat4( red,     0,
                 green,   0,
                 blue,    0,
                 0, 0, 0, 1 );
}

vec2 img2tex( vec2 v ) { return v / textureSize * imageSize; }

void main() {
    vec2 uv = gl_FragCoord.xy / imageSize.xy;

    vec4 col = texture2D(sampler0, img2tex(fract(uv)));

    // vec4 tint = vec4(1,1,1.2,1);

	gl_FragColor = brightnessMatrix(brightness) *
                contrastMatrix(1+brightness) *
                // tint *
        		col;
}